/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 30/05/22.
//

#include "fggl/debug/draw.hpp"
#include "fggl/gfx/ogl/types.hpp"
#include "fggl/gfx/ogl4/debug.hpp"

#include <cassert>
#include <utility>

namespace fggl::gfx::ogl4 {

	DebugRenderer::DebugRenderer(std::shared_ptr<ogl::Shader> shader) :
		mvpMatrix(1.0F),
		m_lineShader(std::move(shader)),
		m_lineShaderMVP(m_lineShader->uniform("u_MvpMatrix")) {
		// define our attributes
		auto posAttr = ogl::attribute<dd::DrawVertex, math::vec3>(0);
		auto colAttr = ogl::attribute<dd::DrawVertex, math::vec3>(sizeof(float) * 3);

		// bind the attributes to the vao
		m_lineVao.setAttribute(m_lineVbo, 0, posAttr);
		m_lineVao.setAttribute(m_lineVbo, 1, colAttr);
	}

	void DebugRenderer::drawLineList(const dd::DrawVertex *lines, int count, bool depthEnabled) {
		assert(lines != nullptr);
		assert(count > 0 && count <= DEBUG_DRAW_VERTEX_BUFFER_SIZE);

		m_lineVao.bind();
		m_lineShader->use();
		m_lineShader->setUniformMtx(m_lineShaderMVP, mvpMatrix);

		if (depthEnabled) {
			glEnable(GL_DEPTH_TEST);
		} else {
			glDisable(GL_DEPTH_TEST);
		}

		m_lineVbo.bind();
		m_lineVbo.replace<dd::DrawVertex>(count, lines);

		m_lineVao.draw(ogl::Primative::LINE, 0, count);

		glUseProgram(0);
		glBindVertexArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}

} // namespace fggl::gfx::ogl4