/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_GFX_OGL4_SETUP_HPP
#define FGGL_GFX_OGL4_SETUP_HPP

#include "fggl/gfx/setup.hpp"
#include "fggl/gfx/ogl/renderer.hpp"

namespace fggl::gfx::ogl4 {

	constexpr GraphicsDetails OPENGL_4_3_PRODUCTION { GraphicsAPI::OpenGL, 4, 3, false };
	constexpr GraphicsDetails OPENGL_4_3_DEBUG { GraphicsAPI::OpenGL, 4, 3, true };

	class WindowGraphics : public gfx::WindowGraphics {
		public:
			WindowGraphics(data::Storage *storage, gui::FontLibrary *fonts, assets::AssetManager* assets) : m_storage(storage), m_fonts(fonts), m_assets(assets) {};
			virtual ~WindowGraphics() = default;

			fggl::gfx::Graphics *create(display::Window &window) override;

			[[nodiscard]]
			inline GraphicsDetails config() const override {
				#ifdef NDEBUG
					return OPENGL_4_3_PRODUCTION;
				#else
					return OPENGL_4_3_DEBUG;
				#endif
			}

		private:
			data::Storage *m_storage;
			gui::FontLibrary *m_fonts;
			assets::AssetManager *m_assets;
	};

} // namespace fggl::gfx::ogl4

#endif //FGGL_GFX_OGL4_SETUP_HPP
