/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_INPUT_KEYBOARD_HPP
#define FGGL_INPUT_KEYBOARD_HPP

#include <array>
#include <vector>
#include <unordered_set>
#include <iostream>

namespace fggl::input {
	using scancode_t = int;

	class KeyboardState {
		public:
			KeyboardState() = default;

			inline void clear() {
				m_keys.clear();
			}

			inline void set(scancode_t code, bool state) {
				if (state) {
					m_keys.insert(code);
				} else {
					m_keys.erase(code);
				}
			}

			inline bool down(scancode_t scancode) const {
				if (m_keys.empty())
					return false;

				return m_keys.count(scancode) > 0;
			}

		private:
			std::unordered_set<scancode_t> m_keys;
	};

	class KeyboardInput {
		public:
			inline void frame(float /*dt*/) {
				m_prev = m_curr;
			}

			inline void set(scancode_t scancode, bool state) {
				m_curr.set(scancode, state);
			}

			inline bool down(scancode_t scancode) const {
				return m_curr.down(scancode);
			}

			inline bool pressed(scancode_t scancode) const {
				return m_curr.down(scancode) && !m_prev.down(scancode);
			}

			inline bool released(scancode_t scancode) const {
				return !m_curr.down(scancode) && m_prev.down(scancode);
			}

		private:
			KeyboardState m_curr;
			KeyboardState m_prev;
	};

};

#endif
